/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.Collection;

import org.apache.commons.lang3.reflect.ConstructorUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.Link;
import org.springframework.stereotype.Component;

import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.BeanProperty;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.ser.ContextualSerializer;

import de.ozgcloud.alfa.common.user.UserManagerProperties;
import de.ozgcloud.alfa.common.user.UserManagerUrlProvider;
import de.ozgcloud.common.errorhandling.TechnicalException;
import lombok.RequiredArgsConstructor;

@Component
@RequiredArgsConstructor(onConstructor_ = { @Autowired })
public class LinkedUserProfileResourceSerializer extends JsonSerializer<Object> implements ContextualSerializer {

	private final UserManagerUrlProvider userManagerUrlProvider;
	private LinkedUserProfileResource annotation;

	// ObjectMapper uses this, if called outside of Spring, so dependency injection does not work
	private LinkedUserProfileResourceSerializer() {
		this(new UserManagerUrlProvider(new UserManagerProperties()));
	}

	@Override
	public JsonSerializer<?> createContextual(SerializerProvider prov, BeanProperty property) {
		return LinkedUserProfileResourceSerializer.createForAnnotatedField(userManagerUrlProvider,
				property.getAnnotation(LinkedUserProfileResource.class));
	}

	static LinkedUserProfileResourceSerializer createForAnnotatedField(UserManagerUrlProvider userManagerUrlProvider,
			LinkedUserProfileResource annotation) {
		var serializer = new LinkedUserProfileResourceSerializer(userManagerUrlProvider);
		serializer.annotation = annotation;
		return serializer;
	}

	@Override
	public void serialize(Object value, JsonGenerator gen, SerializerProvider serializers) throws IOException {
		if (value instanceof Collection) {
			gen.writeStartArray();
			((Collection<?>) value).forEach(val -> writeObject(gen, buildLink(val)));
			gen.writeEndArray();
		} else {
			writeObject(gen, buildLink(value));
		}
	}

	private String buildLink(Object id) {
		return userManagerUrlProvider.isConfiguredForUserProfile() ? Link.of(buildUserProfileUri(id.toString())).getHref() : String.valueOf(id);
	}

	private String buildUserProfileUri(String id) {
		return String.format(userManagerUrlProvider.getUserProfileTemplate(), getExtractor().extractId(id));
	}

	private IdExtractor<Object> getExtractor() {
		try {
			return ConstructorUtils.invokeConstructor(annotation.extractor());
		} catch (NoSuchMethodException | IllegalAccessException | InvocationTargetException | InstantiationException e) {
			throw new TechnicalException("Error instanciating IdExtractor", e);
		}
	}

	private void writeObject(JsonGenerator gen, Object value) {
		try {
			gen.writeObject(value);
		} catch (IOException e) {
			throw new TechnicalException("Error writing String to json", e);
		}
	}
}