/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.loeschanforderung;

import java.util.HashSet;
import java.util.Set;
import java.util.stream.Stream;

import org.apache.commons.collections.MapUtils;
import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItem;
import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItemService;
import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.common.command.CommandService;
import de.ozgcloud.alfa.common.command.CommandStatus;
import de.ozgcloud.alfa.historie.HistorieProcessor;
import lombok.RequiredArgsConstructor;

@Component
@RequiredArgsConstructor
class LoeschAnforderungHistorieProcessor implements HistorieProcessor {

	private final CommandService commandService;
	private final VorgangAttachedItemService vorgangAttachedItemService;

	@Override
	public Stream<Command> process(Stream<Command> commands) {
		var commandIdsToRemove = new HashSet<String>();
		var reducedCommands = commands.filter(this::isNotRevokedLoeschAnforderungZuruecknehmen)
				.filter(this::isNotCreateLoeschAnforderungVorgangAttachedItem)
				.map(command -> addLoeschAnforderungZuruecknehmenRelatedCommandIds(commandIdsToRemove, command))
				.toList();
		return reducedCommands.stream()
				.filter(command -> !commandIdsToRemove.contains(command.getId()));
	}

	boolean isNotRevokedLoeschAnforderungZuruecknehmen(Command command) {
		if (!isLoeschAnforderungZuruecknehmen(command)) {
			return true;
		}
		var changeStatusCommand = commandService.getById(getChangeStatusCommandId(command));
		return CommandStatus.REVOKED != changeStatusCommand.getStatus();
	}

	boolean isNotCreateLoeschAnforderungVorgangAttachedItem(Command command) {
		if (!isCreateAttachedItem(command)) {
			return true;
		}
		return !vorgangAttachedItemService.isLoeschAnforderung(MapUtils.getString(command.getBody(), VorgangAttachedItem.FIELD_ITEM_NAME));
	}

	private boolean isCreateAttachedItem(Command command) {
		return CommandOrder.CREATE_ATTACHED_ITEM.equals(command.getCommandOrder());
	}

	Command addLoeschAnforderungZuruecknehmenRelatedCommandIds(Set<String> commandIdsToRemove, Command command) {
		if (isLoeschAnforderungZuruecknehmen(command)) {
			commandIdsToRemove.addAll(getCommandBodyIds(command));
		}
		return command;
	}

	private boolean isLoeschAnforderungZuruecknehmen(Command command) {
		return CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN.equals(command.getCommandOrder());

	}

	private Set<String> getCommandBodyIds(Command command) {
		return Set.of(getDeleteAttachedItemCommandId(command), getChangeStatusCommandId(command));
	}

	private String getDeleteAttachedItemCommandId(Command command) {
		return MapUtils.getString(command.getBody(), DeleteLoeschAnforderung.DELETE_ATTACHED_ITEM_COMMAND_ID_FIELD);
	}

	String getChangeStatusCommandId(Command command) {
		return MapUtils.getString(command.getBody(), DeleteLoeschAnforderung.CHANGE_STATUS_COMMAND_ID_FIELD);
	}

}
