/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.postfach;

import java.util.Map;
import java.util.stream.Stream;

import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandBodyMapper;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.historie.HistorieProcessor;

@Component
class PostfachNachrichtHistorieProcessor implements HistorieProcessor {

	static final String DIRECTION_INCOMMING = "IN";
	static final String DIRECTION_OUTGOING = "OUT";
	static final String DIRECTION = "direction";
	static final String OZGCLOUD_NACHRICHTEN_MANAGER = "OzgCloud_NachrichtenManager";
	static final String CLIENT = "client";

	@Override
	public Stream<Command> process(Stream<Command> commands) {
		return commands
				.filter(this::isNotOutgoingPostfachNachrichtByNachrichtenManager)
				.map(this::translateSendPostfachMailCommand)
				.map(this::translateReceivePostfachNachrichtCommand);
	}

	boolean isNotOutgoingPostfachNachrichtByNachrichtenManager(Command command) {
		return !(isCreateAttachedItem(command) && isOzgCloudNachrichtenManager(command) && isOutgoing(command));
	}

	private boolean isOutgoing(Command command) {
		return isDirection(command, DIRECTION_OUTGOING);
	}

	public Command translateSendPostfachMailCommand(Command command) {
		if (CommandOrder.SEND_POSTFACH_MAIL == command.getCommandOrder()) {
			return command.toBuilder().order(CommandOrder.SEND_POSTFACH_NACHRICHT.name()).build();
		}
		return command;
	}

	public Command translateReceivePostfachNachrichtCommand(Command command) {
		if (isIncomingNachrichtByNachrichtenManager(command)) {
			return command.toBuilder().order(CommandOrder.RECEIVE_POSTFACH_NACHRICHT.name()).build();
		}
		return command;
	}

	boolean isIncomingNachrichtByNachrichtenManager(Command command) {
		return isCreateAttachedItem(command) && isOzgCloudNachrichtenManager(command) && isIncomming(command);
	}

	private boolean isCreateAttachedItem(Command command) {
		return command.getCommandOrder().equals(CommandOrder.CREATE_ATTACHED_ITEM);
	}

	private boolean isOzgCloudNachrichtenManager(Command command) {
		return StringUtils.equals(MapUtils.getString(command.getBody(), CLIENT), OZGCLOUD_NACHRICHTEN_MANAGER);
	}

	private boolean isIncomming(Command command) {
		return isDirection(command, DIRECTION_INCOMMING);
	}

	private boolean isDirection(Command command, String expectedDirection) {
		return StringUtils.equals(MapUtils.getString(getItemMap(command), DIRECTION), expectedDirection);
	}

	@SuppressWarnings("unchecked")
	private Map<String, Object> getItemMap(Command command) {
		return MapUtils.getMap(command.getBody(), CommandBodyMapper.ITEM_PROPERTY);
	}
}
