/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.postfach;

import java.util.Objects;
import java.util.Optional;

import org.springframework.cloud.endpoint.RefreshEndpoint;
import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.FeatureToggleProperties;
import de.ozgcloud.alfa.vorgang.ServiceKonto;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Service
class PostfachSettingsService {

	static final String FIELD_SIGNATUR = "signatur";

	private PostfachConfigGroup postfachConfigGroup;

	private final PostfachMailRemoteService remoteService;
	private final RefreshEndpoint refreshEndpoint;
	private final PostfachProperties postfachProperties;
	private final FeatureToggleProperties featureToggleProperties;

	public boolean isPostfachConfigured() {
		if (Objects.isNull(postfachConfigGroup)) {
			postfachConfigGroup = remoteService.getPostfachConfig();
		}
		return postfachConfigGroup.isConfigured();
	}

	public PostfachSettings getPostfachSettings(VorgangWithEingang vorgang) {
		return PostfachSettings.builder()
				.features(Features.builder().reply(isReplyToMessageAllowed(vorgang)).build())
				.settings(Settings.builder().signatur(getSignatur()).build())
				.build();
	}

	String getSignatur() {
		refreshPostfachProperties();
		return postfachProperties.getSignatur();
	}

	void refreshPostfachProperties() {
		refreshEndpoint.refresh();
	}

	boolean isReplyToMessageAllowed(VorgangWithEingang vorgang) {
		return Optional.ofNullable(vorgang.getHeader().getServiceKonto())
				.map(ServiceKonto::getType)
				.map(this::isReplyAllowed)
				.orElse(false);
	}

	boolean isReplyAllowed(String serviceKontoType) {
		if (featureToggleProperties.isReplyAlwaysAllowed()) {
			return true;
		}

		if (!isPostfachConfigured()) {
			return false;
		}

		return postfachConfigGroup.getPostfachConfigs().stream()
				.filter(postfachConfig -> postfachConfig.getType().equalsIgnoreCase(serviceKontoType))
				.map(PostfachConfig::isReplyAllowed)
				.findFirst()
				.orElse(false);
	}
}
