/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.vorgang.command;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.util.Objects;

import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.LinkRelation;
import org.springframework.hateoas.server.RepresentationModelProcessor;
import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.common.command.CommandController.CommandByRelationController;
import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserRole;
import de.ozgcloud.alfa.vorgang.Vorgang;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;
import de.ozgcloud.alfa.vorgang.VorgangController;
import lombok.RequiredArgsConstructor;

@Component
@RequiredArgsConstructor
class VorgangCommandProcessor implements RepresentationModelProcessor<EntityModel<? extends Vorgang>> {

	static final LinkRelation REL_VORGANG_ANNEHMEN = LinkRelation.of("annehmen");
	static final LinkRelation REL_VORGANG_VERWERFEN = LinkRelation.of("verwerfen");
	static final LinkRelation REL_VORGANG_ZURUECKHOLEN = LinkRelation.of("zurueckholen");
	static final LinkRelation REL_VORGANG_BEARBEITEN = LinkRelation.of("bearbeiten");
	static final LinkRelation REL_VORGANG_BESCHEIDEN = LinkRelation.of("bescheiden");
	static final LinkRelation REL_VORGANG_ZURUECKSTELLEN = LinkRelation.of("zurueckstellen");
	static final LinkRelation REL_VORGANG_ABSCHLIESSEN = LinkRelation.of("abschliessen");
	static final LinkRelation REL_VORGANG_WIEDEREROEFFNEN = LinkRelation.of("wiedereroeffnen");

	private final CurrentUserService userService;
	private final VorgangController vorgangController;

	@Override
	public EntityModel<? extends Vorgang> process(EntityModel<? extends Vorgang> model) {
		var vorgang = model.getContent();

		if (Objects.isNull(vorgang) || !vorgangController.isEditable(vorgang)) {
			return model;
		}

		addLinkIfTransactionAllowed(VorgangStatus.ANGENOMMEN, getRelationForAngenommenByStatus(vorgang), vorgang, model);
		addLinkIfTransactionAllowed(VorgangStatus.VERWORFEN, REL_VORGANG_VERWERFEN, vorgang, model);
		addLinkIfTransactionAllowed(VorgangStatus.NEU, REL_VORGANG_ZURUECKHOLEN, vorgang, model);
		addLinkIfTransactionAllowed(VorgangStatus.IN_BEARBEITUNG, getRelationForInBearbeitungByStatus(vorgang), vorgang, model);
		addLinkIfTransactionAllowed(VorgangStatus.BESCHIEDEN, REL_VORGANG_BESCHEIDEN, vorgang, model);
		addLinkIfTransactionAllowed(VorgangStatus.ABGESCHLOSSEN, REL_VORGANG_ABSCHLIESSEN, vorgang, model);

		return model;
	}

	private LinkRelation getRelationForAngenommenByStatus(Vorgang vorgang) {
		if (vorgang.getStatus() == VorgangStatus.NEU) {
			return REL_VORGANG_ANNEHMEN;
		} else {
			return REL_VORGANG_ZURUECKSTELLEN;
		}
	}

	private LinkRelation getRelationForInBearbeitungByStatus(Vorgang vorgang) {
		if (vorgang.getStatus() == VorgangStatus.ANGENOMMEN) {
			return REL_VORGANG_BEARBEITEN;
		} else {
			return REL_VORGANG_WIEDEREROEFFNEN;
		}
	}

	private void addLinkIfTransactionAllowed(VorgangStatus newStatus, LinkRelation relation, Vorgang vorgang, EntityModel<? extends Vorgang> model) {
		if (isTransactonAllowed(vorgang, newStatus)) {
			addCreateCommandLink(relation, vorgang, model);
		}
	}

	private void addCreateCommandLink(LinkRelation relation, Vorgang vorgang, EntityModel<? extends Vorgang> model) {
		model.add(linkTo(methodOn(CommandByRelationController.class)
				.createCommand(vorgang.getId(), vorgang.getId(), vorgang.getVersion(), null)).withRel(relation));
	}

	private boolean isTransactonAllowed(Vorgang vorgang, VorgangStatus newStatus) {
		return vorgang.getStatus().isTransactionAllowed(getUserRole(), newStatus);
	}

	String getUserRole() {
		String role;
		if (userService.hasRole(UserRole.EINHEITLICHER_ANSPRECHPARTNER)) {
			role = UserRole.EINHEITLICHER_ANSPRECHPARTNER;
		} else if (userService.hasRole(UserRole.VERWALTUNG_POSTSTELLE)) {
			role = UserRole.VERWALTUNG_POSTSTELLE;
		} else {
			role = UserRole.VERWALTUNG_USER;
		}

		return role;
	}

}