/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.bescheid;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.util.List;
import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import de.ozgcloud.alfa.common.binaryfile.BinaryFileController;
import de.ozgcloud.alfa.common.file.OzgFileTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import lombok.SneakyThrows;

class BescheidControllerTest {

	@Spy
	@InjectMocks
	private BescheidController controller;
	@Mock
	private BescheidModelAssembler assembler;
	@Mock
	private BescheidService bescheidService;
	@Mock
	private BinaryFileController binaryFileController;

	private MockMvc mockMvc;

	private final Bescheid draft = BescheidTestFactory.create();

	@BeforeEach
	void initTest() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	@Nested
	class TestGetDraft {

		@SneakyThrows
		@Test
		void shouldCallBescheidService() {
			doRequest();

			verify(bescheidService).getBescheidDraft(VorgangHeaderTestFactory.ID);
		}

		@SneakyThrows
		@Test
		void shouldReturnStatusNotFoundIfDraftDoesNotExist() {
			when(bescheidService.getBescheidDraft(BescheidTestFactory.VORGANG_ID)).thenReturn(Optional.empty());

			var response = doRequest();

			response.andExpect(status().isNotFound());
		}

		@DisplayName("on existing")
		@Nested
		class TestOnExistingDraft {

			@BeforeEach
			void mock() {
				when(bescheidService.getBescheidDraft(BescheidTestFactory.VORGANG_ID)).thenReturn(Optional.of(draft));
			}

			@SneakyThrows
			@Test
			void shouldCallAssembler() {
				doRequest();

				verify(assembler).toModel(draft);
			}

			@SneakyThrows
			@Test
			void shouldReturnValue() {
				when(assembler.toModel(draft)).thenReturn(EntityModel.of(draft));

				doRequest().andExpect(status().isOk());
			}
		}

		@SneakyThrows
		private ResultActions doRequest() {
			return mockMvc.perform(get(BescheidController.PATH).param("vorgangId", VorgangHeaderTestFactory.ID));
		}
	}

	@DisplayName("Get Attachments")
	@Nested
	class TestGetAttachments {

		@BeforeEach
		void mock() {
			when(bescheidService.getBescheid(any())).thenReturn(BescheidTestFactory.create());
			when(binaryFileController.getFiles(any())).thenReturn(CollectionModel.of(List.of(EntityModel.of(OzgFileTestFactory.create()))));
		}

		@SneakyThrows
		@Test
		void shouldCallBescheidService() {
			doRequest();

			verify(bescheidService).getBescheid(BescheidTestFactory.ID);
		}

		@SneakyThrows
		@Test
		void shouldCallBinaryFileController() {
			doRequest();

			verify(binaryFileController).getFiles(BescheidTestFactory.ATTACHMENTS);
		}

		@SneakyThrows
		@Test
		void shouldReturnStatusOk() {
			var response = doRequest();

			response.andExpect(status().isOk());
		}

		@SneakyThrows
		@Test
		void shouldHaveAttachmentName() {
			var response = doRequest();

			response.andExpect(jsonPath("$.content[0].name").value(OzgFileTestFactory.NAME));
		}

		@SneakyThrows
		@Test
		void shouldHaveAttachmentSize() {
			var response = doRequest();

			response.andExpect(jsonPath("$.content[0].size").value(OzgFileTestFactory.SIZE));
		}

		@SneakyThrows
		@Test
		void shouldHaveAttachmentContentType() {
			var response = doRequest();

			response.andExpect(jsonPath("$.content[0].contentType").value(OzgFileTestFactory.CONTENT_TYPE));
		}

		@SneakyThrows
		private ResultActions doRequest() {
			return mockMvc.perform(
					get(BescheidController.PATH + "/{bescheidId}/attachments", BescheidTestFactory.ID).param("vorgangId",
							BescheidTestFactory.VORGANG_ID));
		}
	}
}