/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.bescheid;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.document.bescheid.BescheidServiceGrpc.BescheidServiceBlockingStub;
import de.ozgcloud.document.bescheid.GrpcBescheidManagerConfigRequest;
import de.ozgcloud.document.bescheid.GrpcBescheidManagerConfigResponse;
import de.ozgcloud.document.bescheid.GrpcGetAllBescheidRequest;
import de.ozgcloud.document.bescheid.GrpcGetAllBescheidResponse;
import de.ozgcloud.document.bescheid.GrpcGetBescheidDraftRequest;
import de.ozgcloud.document.bescheid.GrpcGetBescheidDraftResponse;
import de.ozgcloud.document.bescheid.GrpcGetBescheidRequest;
import de.ozgcloud.document.bescheid.GrpcGetBescheidResponse;

class BescheidRemoteServiceTest {

	@Spy
	@InjectMocks
	private BescheidRemoteService service;
	@Mock
	private BescheidServiceBlockingStub bescheidServiceStub;
	@Mock
	private BescheidMapper bescheidMapper;
	@Mock
	private BescheidManagerFeaturesMapper featuresMapper;

	@Nested
	class TestGetBescheidDraft {

		private final GrpcGetBescheidDraftRequest request = GrpcGetBescheidDraftRequestTestFactory.create();
		private final GrpcGetBescheidDraftResponse response = GrpcGetBescheidDraftResponseTestFactory.create();
		private final Bescheid bescheidDraft = BescheidTestFactory.create();

		@BeforeEach
		void setUp() {
			doReturn(request).when(service).buildGetBescheidDraftRequest(VorgangHeaderTestFactory.ID);
			when(bescheidServiceStub.getBescheidDraft(request)).thenReturn(response);
		}

		@Test
		void shouldBuildGrpcGetBescheidRequest() {
			service.getBescheidDraft(VorgangHeaderTestFactory.ID);

			verify(service).buildGetBescheidDraftRequest(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldCallGrpcService() {
			service.getBescheidDraft(VorgangHeaderTestFactory.ID);

			verify(bescheidServiceStub).getBescheidDraft(request);
		}

		@Test
		void shouldGetBescheidFromResponse() {
			service.getBescheidDraft(VorgangHeaderTestFactory.ID);

			verify(service).getBescheidFromResponse(response);
		}

		@Test
		void shouldCallMapper() {
			doReturn(Optional.of(GrpcGetBescheidDraftResponseTestFactory.GRPC_BESCHEID)).when(service).getBescheidFromResponse(response);

			service.getBescheidDraft(VorgangHeaderTestFactory.ID);

			verify(bescheidMapper).fromGrpcAndVorgangId(GrpcGetBescheidDraftResponseTestFactory.GRPC_BESCHEID, VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldReturnBescheid() {
			when(bescheidMapper.fromGrpcAndVorgangId(GrpcGetBescheidDraftResponseTestFactory.GRPC_BESCHEID, VorgangHeaderTestFactory.ID)).thenReturn(
					bescheidDraft);

			var bescheidDraft = service.getBescheidDraft(VorgangHeaderTestFactory.ID);

			assertThat(bescheidDraft).hasValue(this.bescheidDraft);
		}

		@Test
		void shouldReturnEmpty() {
			doReturn(Optional.empty()).when(service).getBescheidFromResponse(response);

			var bescheidDraft = service.getBescheidDraft(VorgangHeaderTestFactory.ID);

			assertThat(bescheidDraft).isEmpty();
		}
	}

	@Nested
	class TestBuildGetBescheidDraftRequest {

		@Test
		void shouldHaveVorgangId() {
			var request = service.buildGetBescheidDraftRequest(VorgangHeaderTestFactory.ID);

			assertThat(request.getVorgangId()).isEqualTo(VorgangHeaderTestFactory.ID);
		}
	}

	@Nested
	class TestGetBescheidFromResponse {

		@Test
		void shouldReturnEmpty() {
			var bescheid = service.getBescheidFromResponse(GrpcGetBescheidDraftResponse.getDefaultInstance());

			assertThat(bescheid).isEmpty();
		}

		@Test
		void shouldReturnBescheid() {
			var bescheid = service.getBescheidFromResponse(GrpcGetBescheidDraftResponseTestFactory.create());

			assertThat(bescheid).hasValue(GrpcGetBescheidDraftResponseTestFactory.GRPC_BESCHEID);
		}
	}

	@Nested
	class TestGetBescheid {

		private final GrpcGetBescheidRequest request = GrpcGetBescheidRequestTestFactory.create();
		private final GrpcGetBescheidResponse response = GrpcGetBescheidResponseTestFactory.create();

		@BeforeEach
		void setUp() {
			doReturn(request).when(service).buildGetBescheidRequest(any());
			when(bescheidServiceStub.getBescheid(any())).thenReturn(response);
		}

		@Test
		void shouldBuildGetBescheidRequest() {
			service.getBescheid(BescheidTestFactory.ID);

			verify(service).buildGetBescheidRequest(BescheidTestFactory.ID);
		}

		@Test
		void shouldCallGrpcService() {
			service.getBescheid(BescheidTestFactory.ID);

			verify(bescheidServiceStub).getBescheid(request);
		}

		@Test
		void shouldCallMapper() {
			service.getBescheid(BescheidTestFactory.ID);

			verify(bescheidMapper).fromGrpc(GrpcGetBescheidResponseTestFactory.GRPC_BESCHEID);
		}

		@Test
		void shouldReturnBescheid() {
			var mappedBescheid = BescheidTestFactory.create();
			when(bescheidMapper.fromGrpc(GrpcGetBescheidDraftResponseTestFactory.GRPC_BESCHEID)).thenReturn(mappedBescheid);

			var bescheid = service.getBescheid(BescheidTestFactory.ID);

			assertThat(bescheid).isEqualTo(mappedBescheid);
		}
	}

	@Nested
	class TestBuildGetBescheidRequest {

		@Test
		void shouldHaveId() {
			var request = service.buildGetBescheidRequest(BescheidTestFactory.ID);

			assertThat(request.getId()).isEqualTo(BescheidTestFactory.ID);
		}
	}

	@Nested
	class TestGetBescheidManagerFeatures {

		private final GrpcBescheidManagerConfigRequest request = GrpcBescheidManagerConfigRequestTestFactory.create();
		private final GrpcBescheidManagerConfigResponse respone = GrpcBescheidManagerConfigResponseTestFactory.create();
		private final BescheidManagerFeatures mappedFeatures = BescheidManagerFeatures.builder().build();

		@BeforeEach
		void setUp() {
			when(bescheidServiceStub.getConfig(request)).thenReturn(respone);
			when(featuresMapper.fromGrpc(any())).thenReturn(mappedFeatures);
		}

		@Test
		void shouldCallGrpcService() {
			service.getBescheidManagerFeatures();

			verify(bescheidServiceStub).getConfig(request);
		}

		@Test
		void shouldCallMapper() {
			service.getBescheidManagerFeatures();

			verify(featuresMapper).fromGrpc(GrpcBescheidManagerFeaturesTestFactory.create());
		}

		@Test
		void shouldReturnMappedFeatures() {
			var features = service.getBescheidManagerFeatures();

			assertThat(features).isSameAs(mappedFeatures);
		}
	}

	@DisplayName("Find by vorgang id")
	@Nested
	class TestFindByVorgangId {

		private final GrpcGetAllBescheidRequest request = GrpcGetAllBescheidRequestTestFactory.create();
		private final GrpcGetAllBescheidResponse response = GrpcGetAllBescheidResponseTestFactory.create();

		private final Bescheid bescheid = BescheidTestFactory.create();

		@BeforeEach
		void setUp() {
			doReturn(request).when(service).buildGetAllBescheidRequest(VorgangHeaderTestFactory.ID);
			when(bescheidServiceStub.getAll(any())).thenReturn(response);

		}

		@Test
		void shouldBuildGetAllRequest() {
			service.findByVorgangId(VorgangHeaderTestFactory.ID);

			verify(service).buildGetAllBescheidRequest(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldCallRemoteService() {
			service.findByVorgangId(VorgangHeaderTestFactory.ID);

			verify(bescheidServiceStub).getAll(request);
		}

		@Test
		void shouldMapBescheid() {
			service.findByVorgangId(VorgangHeaderTestFactory.ID).toList();

			verify(bescheidMapper).fromGrpcAndVorgangId(GrpcGetAllBescheidResponseTestFactory.GRPC_BESCHEID, VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldReturnStreamOfBescheide() {
			when(bescheidMapper.fromGrpcAndVorgangId(any(), any())).thenReturn(bescheid);

			var bescheide = service.findByVorgangId(VorgangHeaderTestFactory.ID);

			assertThat(bescheide).containsExactly(bescheid);
		}
	}

	@DisplayName("Build get all bescheid request")
	@Nested
	class TestBuildGetAllBescheidRequest {

		@Test
		void shouldContainVorgangId() {
			var request = service.buildGetAllBescheidRequest(VorgangHeaderTestFactory.ID);

			assertThat(request.getVorgangId()).isEqualTo(VorgangHeaderTestFactory.ID);
		}
	}
}