package de.ozgcloud.alfa.collaboration;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.EnumSource;
import org.junit.jupiter.params.provider.EnumSource.Mode;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;

import de.ozgcloud.alfa.common.FeatureToggleProperties;
import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserRole;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;

public class OrganisationsEinheitVorgangWithEingangProcessorTest {

	@InjectMocks
	@Spy
	private OrganisationsEinheitVorgangWithEingangProcessor processor;

	@Mock
	private CurrentUserService userService;
	@Mock
	private FeatureToggleProperties featureToggleProperties;

	@DisplayName("Process")
	@Nested
	class TestProcess {

		private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();

		private final EntityModel<VorgangWithEingang> model = EntityModel.of(vorgang);

		@Test
		void shouldReturnSameModel() {
			var processedModel = processor.process(model);

			assertThat(processedModel).isSameAs(model);
		}

		@SuppressWarnings("unchecked")
		@Test
		void shouldReturnSameModelOnNullContent() {
			var entityModel = mock(EntityModel.class);
			when(entityModel.getContent()).thenReturn(null);

			var processedModel = processor.process(entityModel);

			assertThat(processedModel).isSameAs(entityModel);
		}

		@DisplayName("entity model links")
		@Nested
		class TestEntityModelLinks {

			@Test
			void shouldContainsSearchLinkIfNeeded() {
				doReturn(true).when(processor).isSearchNeeded(any());

				var processedModel = processor.process(model);

				assertThat(processedModel.getLink(OrganisationsEinheitVorgangWithEingangProcessor.REL_SEARCH_ORGANISATIONS_EINHEIT)).get()
						.extracting(Link::getHref)
						.isEqualTo(OrganisationsEinheitController.PATH + "?searchBy={searchBy}");
			}
		}
	}

	@DisplayName("is search needed")
	@Nested
	class TestIsSearchNeeded {

		private VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();

		@Test
		void shouldReturnTrueIfForwardable() {
			doReturn(true).when(processor).isForwardable(any());

			var isSearchNeeded = processor.isSearchNeeded(vorgang);

			assertThat(isSearchNeeded).isTrue();
		}

		@Test
		void shouldReturnFalseIfCollaborationCreateable() {
			doReturn(true).when(processor).isCollaborationCreateable();

			var isSearchNeeded = processor.isSearchNeeded(vorgang);

			assertThat(isSearchNeeded).isTrue();
		}
	}

	@DisplayName("is forwardable")
	@Nested
	class TestIsForwardable {

		@DisplayName("on feature disabled")
		@Nested
		class TestOnFeatureDisabled {

			@BeforeEach
			void mockFeatureToggle() {
				when(featureToggleProperties.isForwardByOzgCloudEnabled()).thenReturn(false);
			}

			@ParameterizedTest
			@EnumSource
			void shouldReturnFalse(VorgangStatus status) {
				var vorgang = VorgangWithEingangTestFactory.createBuilder().status(status).build();

				var forwardableByOzgCloud = processor.isForwardable(vorgang);

				assertThat(forwardableByOzgCloud).isFalse();
			}
		}

		@DisplayName("on feature enabled")
		@Nested
		class TestOnFeatureEnabled {

			@BeforeEach
			void mockFeatureToggle() {
				when(featureToggleProperties.isForwardByOzgCloudEnabled()).thenReturn(true);
			}

			@ParameterizedTest(name = "{0}")
			@EnumSource(mode = Mode.EXCLUDE, names = { "NEU" })
			void shouldReturnFalseOnVorgangStatus(VorgangStatus status) {
				var forwardableByOzgCloud = processor.isForwardable(VorgangWithEingangTestFactory.createBuilder().status(status).build());

				assertThat(forwardableByOzgCloud).isFalse();
			}

			@Test
			void shouldReturnTrueOnVorgangStatusNeu() {
				var forwardable = processor.isForwardable(VorgangWithEingangTestFactory.createBuilder().status(VorgangStatus.NEU).build());

				assertThat(forwardable).isTrue();
			}
		}
	}

	@DisplayName("is collaboration createable")
	@Nested
	class TestIsCollaborationCreateable {

		@DisplayName("on feature disabled")
		@Nested
		class TestOnFeatureDisabled {

			@BeforeEach
			void mockFeatureToggle() {
				when(featureToggleProperties.isCollaborationEnabled()).thenReturn(false);
			}

			void shouldReturnFalse() {
				var isCollaborationEnabled = processor.isCollaborationCreateable();

				assertThat(isCollaborationEnabled).isFalse();
			}
		}

		@DisplayName("on feature enabled")
		@Nested
		class TestOnFeatureEnabled {

			@BeforeEach
			void mockFeatureToggle() {
				when(featureToggleProperties.isCollaborationEnabled()).thenReturn(true);
			}

			@Test
			void shouldVerifyRole() {
				processor.isCollaborationCreateable();

				verify(userService).hasRole(UserRole.VERWALTUNG_USER);
			}

			@Test
			void shouldReturnFalseMissingRole() {
				when(userService.hasRole(any())).thenReturn(false);

				var isCollaborationEnabled = processor.isCollaborationCreateable();

				assertThat(isCollaborationEnabled).isFalse();
			}

			@Test
			void shouldReturnTrueOnAssignedRole() {
				when(userService.hasRole(any())).thenReturn(true);

				var isCollaborationEnabled = processor.isCollaborationCreateable();

				assertThat(isCollaborationEnabled).isTrue();
			}
		}
	}
}
