/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.command;

import static de.ozgcloud.alfa.common.command.CommandModelAssembler.*;
import static de.ozgcloud.alfa.common.command.CommandTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.ArrayList;
import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.EnumSource;
import org.junit.jupiter.params.provider.EnumSource.Mode;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.IanaLinkRelations;
import org.springframework.hateoas.Link;

import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;

class CommandModelAssemblerTest {

	private static final String COMMAND_SINGLE_PATH = "/api/commands/" + CommandTestFactory.ID;

	@InjectMocks
	private CommandModelAssembler modelAssembler;

	@Spy
	private List<CommandProcessor> processors = new ArrayList<>();
	@Mock
	private CommandProcessor processor;

	@Captor
	private ArgumentCaptor<EntityModel<Command>> commandEntityModelCaptor;

	@BeforeEach
	void setProcessorMocks() {
		processors.add(processor);
	}

	@Test
	void shouldHaveSelfLink() {
		var model = modelAssembler.toModel(CommandTestFactory.create());

		assertThat(model.getLink(IanaLinkRelations.SELF)).isPresent().get().extracting(Link::getHref).isEqualTo(COMMAND_SINGLE_PATH);
	}

	@Test
	void shouldCallProcessor() {
		var command = CommandTestFactory.create();

		modelAssembler.toModel(command);

		verify(processor).process(commandEntityModelCaptor.capture());
		assertThat(commandEntityModelCaptor.getValue().getContent()).isEqualTo(command);
	}

	@Nested
	@DisplayName("Link to effected Resource")
	class TestEffectedResourceLink {

		@DisplayName("on vorgang")
		@Nested
		class TestOnVorgang {

			private static final String VORGANG_URL = "/api/vorgangs/" + RELATION_ID;

			@ParameterizedTest
			@EnumSource(mode = Mode.INCLUDE, names = { "FINISHED", "REVOKED" })
			void shouldBePresentOnStatus(CommandStatus status) {
				var model = toModelWithStatus(status);

				assertThat(model.getLink(REL_EFFECTED_RESOURCE)).isPresent().get().extracting(Link::getHref).isEqualTo(VORGANG_URL);
			}

			@ParameterizedTest
			@EnumSource(mode = Mode.EXCLUDE, names = { "FINISHED", "REVOKED", "ERROR" })
			void shouldNOTbePresentOtherWise(CommandStatus status) {
				var model = toModelWithStatus(status);

				assertThat(model.getLink(REL_EFFECTED_RESOURCE)).isEmpty();
			}

			private EntityModel<Command> toModelWithStatus(CommandStatus status) {
				return modelAssembler.toModel(CommandTestFactory.createBuilder().order(CommandOrder.VORGANG_ANNEHMEN.name()).status(status).build());
			}
		}

		@DisplayName("on forwarding")
		@Nested
		class TestOnForwarding {

			private static final String FORWARDING_URL = "/api/forwardings?vorgangId=" + VorgangHeaderTestFactory.ID;

			@ParameterizedTest
			@ValueSource(strings = { "FORWARD_SUCCESSFULL", "FORWARD_FAILED", "REDIRECT_VORGANG" })
			void shouldHaveLinkToForwarding(String order) {
				var model = toModelWithOrder(order);

				assertThat(model.getLink(CommandModelAssembler.REL_EFFECTED_RESOURCE)).isPresent().get()
						.extracting(Link::getHref).isEqualTo(FORWARDING_URL);
			}
		}

		@DisplayName("on bescheid")
		@Nested
		class TestOnBescheid {

			private static final String BESCHEID_URL = "/api/bescheids?vorgangId=" + VorgangHeaderTestFactory.ID;

			@ParameterizedTest
			@ValueSource(strings = { "CREATE_BESCHEID", "UPDATE_BESCHEID" })
			void shouldHaveLinkToBescheidDokument(String order) {
				var model = modelAssembler
						.toModel(CommandTestFactory.createBuilder().order(order).createdResource(RELATION_ID).build());

				assertThat(model.getLink(CommandModelAssembler.REL_EFFECTED_RESOURCE)).isPresent().get()
						.extracting(Link::getHref).isEqualTo(BESCHEID_URL);
			}
		}

		@DisplayName("on document")
		@Nested
		class TestOnDocument {

			private static final String DOCUMENT_URL = "/api/bescheids/documents/" + RELATION_ID;

			@Test
			void shouldHaveLinkToBescheidDokument() {
				var model = modelAssembler.toModel(
						CommandTestFactory.createBuilder().order(CommandOrder.CREATE_BESCHEID_DOCUMENT_FROM_FILE.name())
								.createdResource(RELATION_ID)
								.build());

				assertThat(model.getLink(CommandModelAssembler.REL_EFFECTED_RESOURCE)).isPresent().get()
						.extracting(Link::getHref).isEqualTo(DOCUMENT_URL);
			}
		}

		@DisplayName("on collaboration")
		@Nested
		class TestOnCollaboration {

			private static final String COLLABORATIONS_URL = "/api/vorgangs/" + VORGANG_ID + "/collaborations";

			@Test
			void shouldHaveLinkToCollaborations() {
				var model = toModelWithOrder("CREATE_COLLABORATION_REQUEST");

				assertThat(model.getLink(CommandModelAssembler.REL_EFFECTED_RESOURCE)).isPresent().get()
						.extracting(Link::getHref).isEqualTo(COLLABORATIONS_URL);
			}
		}

		@DisplayName("on unknown Order")
		@Nested
		class TestOnUnknownOrder {
			@Test
			void shouldNotAddEffectedResourceLink() {
				var model = modelAssembler.toModel(
						CommandTestFactory.createBuilder().order("UnknownOrder")
								.createdResource(RELATION_ID)
								.build());

				assertThat(model.getLink(CommandModelAssembler.REL_EFFECTED_RESOURCE)).isNotPresent();
			}

			@Nested
			class TestEffectedResourceLinkByOrderType {
				@Test
				void shouldThrowIllegalArgumentException() {
					var command = CommandTestFactory.createBuilder().order("UnknownOrder")
							.createdResource(RELATION_ID)
							.build();

					assertThrows(IllegalArgumentException.class, () -> modelAssembler.effectedResourceLinkByOrderType(command));
				}
			}
		}

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = { "UNBEKANNT", "SEND_POSTFACH_MAIL", "SEND_POSTFACH_NACHRICHT", "RESEND_POSTFACH_MAIL",
				"RECEIVE_POSTFACH_NACHRICHT" })
		void shouldBeAbleToBuildLinkForEveryOrder(CommandOrder order) {
			var link = modelAssembler.effectedResourceLinkByOrderType(CommandTestFactory.createBuilder().order(order.name()).build());

			assertThat(link).isNotNull();
		}

		@Test
		void shouldAddLinkForTypeVORGANG_LIST() {
			var link = modelAssembler
					.effectedResourceLinkByOrderType(CommandTestFactory.createBuilder().order(CommandOrder.VORGANG_LOESCHEN.name()).build());

			assertThat(link.getHref()).isEqualTo("/api/vorgangs");
		}
	}

	@DisplayName("Revoke link")
	@Nested
	class TestRevokeLink {

		@DisplayName("related to command")
		@Nested
		class TestRelatedToCommand {

			@ParameterizedTest
			@EnumSource(mode = Mode.EXCLUDE, names = { "REDIRECT_VORGANG", "FORWARD_SUCCESSFULL",
					"FORWARD_FAILED", "ASSIGN_USER", "SEND_POSTFACH_MAIL", "SEND_POSTFACH_NACHRICHT", "RESEND_POSTFACH_MAIL", "CREATE_ATTACHED_ITEM",
					"UPDATE_ATTACHED_ITEM", "PATCH_ATTACHED_ITEM", "RECEIVE_POSTFACH_NACHRICHT", "VORGANG_LOESCHEN", "DELETE_ATTACHED_ITEM",
					"VORGANG_ZUM_LOESCHEN_MARKIEREN", "LOESCH_ANFORDERUNG_ZURUECKNEHMEN", "CREATE_BESCHEID", "PROCESS_VORGANG", "SET_AKTENZEICHEN",
					"DELETE_BESCHEID", "UPDATE_BESCHEID", "CREATE_BESCHEID_DOCUMENT_FROM_FILE", "CREATE_BESCHEID_DOCUMENT", "SEND_BESCHEID",
					"UNBEKANNT", "CREATE_COLLABORATION_REQUEST", "FORWARD_VORGANG" })
			void shouldBePresentOnOrder(CommandOrder order) {
				var model = toModelWithOrder(order.name());

				assertThat(model.getLink(REL_REVOKE)).isPresent().get().extracting(Link::getHref).isEqualTo(COMMAND_SINGLE_PATH);
			}
		}

		@DisplayName("related to loeschAnforderung")
		@Nested
		class TestRelatedToLoeschAnforderung {

			@ParameterizedTest
			@ValueSource(strings = { "VORGANG_ZUM_LOESCHEN_MARKIEREN", "LOESCH_ANFORDERUNG_ZURUECKNEHMEN" })
			void shouldNotBePresentOnOrder(String order) {
				var model = toModelWithOrder(order);

				assertThat(model.getLink(REL_REVOKE)).isNotPresent();
			}
		}

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = { "VORGANG_ANNEHMEN", "VORGANG_VERWERFEN", "VORGANG_ZURUECKHOLEN", "VORGANG_BEARBEITEN",
				"VORGANG_BESCHEIDEN", "VORGANG_ZURUECKSTELLEN", "VORGANG_ABSCHLIESSEN", "VORGANG_WIEDEREROEFFNEN", "VORGANG_ZUM_LOESCHEN_MARKIEREN",
				"LOESCH_ANFORDERUNG_ZURUECKNEHMEN", "FORWARD_VORGANG" })
		void shouldNOTBePresentOnOrder(CommandOrder order) {
			var model = toModelWithOrder(order.name());

			assertThat(model.getLink(REL_REVOKE)).isNotPresent();
		}

		@Test
		void shouldNotHaveLinkIfCommandFailed() {
			var command = CommandTestFactory.createBuilder().order(CommandOrder.VORGANG_ANNEHMEN.name()).status(CommandStatus.ERROR).build();
			var model = modelAssembler.toModel(command);

			assertThat(model.getLink(REL_REVOKE)).isNotPresent();
		}
	}

	private EntityModel<Command> toModelWithOrder(String order) {
		return modelAssembler.toModel(CommandTestFactory.createBuilder().order(order).build());
	}

	@DisplayName("Update link")
	@Nested
	class TestUpdateLink {

		@ParameterizedTest
		@EnumSource(names = { "PENDING", "REVOKE_PENDING" })
		void shouldHave(CommandStatus status) {
			var model = modelAssembler.toModel(CommandTestFactory.createBuilder().status(status).build());

			assertThat(model.getLink(REL_UPDATE)).isPresent().get().extracting(Link::getHref).isEqualTo(COMMAND_SINGLE_PATH);
		}

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = { "PENDING", "REVOKE_PENDING" })
		void shouldNotHaveLink(CommandStatus status) {
			var model = modelAssembler.toModel(CommandTestFactory.createBuilder().status(status).build());

			assertThat(model.getLink(REL_UPDATE)).isNotPresent();
		}
	}
}