package de.ozgcloud.alfa.common.command;

import static org.assertj.core.api.Assertions.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Spy;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.server.mvc.WebMvcLinkBuilder;

class CommandProcessorTest {

	@Spy
	private CommandProcessor processor;

	@DisplayName("Process")
	@Nested
	class TestProcess {

		private final Command command = CommandTestFactory.create();
		private final EntityModel<Command> model = EntityModel.of(command);

		private final WebMvcLinkBuilder linkBuilder = WebMvcLinkBuilder.linkTo(CommandController.class).slash("id");

		@Test
		void shouldNotThrowExceptionsOnNullContent() {
			EntityModel<Command> entityModel = when(mock(EntityModel.class).getContent()).thenReturn(null).getMock();

			assertDoesNotThrow(() -> processor.process(entityModel));
		}

		@DisplayName("effected resource")
		@Nested
		class TestEffectedResource {

			@DisplayName("should verify if responsibility is related to the order")
			@Test
			void shouldCallIsResponsibleForEffectedResource() {
				processor.process(model);

				verify(processor).isResponsibleForEffectedResource(CommandOrder.fromOrder(CommandTestFactory.ORDER));
			}

			@DisplayName("should NOT create link builder for related resource if the responsibility matches")
			@Test
			void shouldNOTCallCreateEffectedResourceLinkBuilder() {
				when(processor.isResponsibleForEffectedResource(any())).thenReturn(false);

				processor.process(model);

				verify(processor, never()).createEffectedResourceLinkBuilder(command);
			}

			@DisplayName("should create link builder for effected resource if the responsibility matches")
			@Test
			void shouldCallCreateEffectedResourceLinkBuilder() {
				when(processor.isResponsibleForEffectedResource(any())).thenReturn(true);
				doReturn(linkBuilder).when(processor).createEffectedResourceLinkBuilder(any());

				processor.process(model);

				verify(processor).createEffectedResourceLinkBuilder(command);
			}

			@DisplayName("link")
			@Nested
			class TestEffectedResourceLink {

				@Test
				void shouldBeAddedIfResponsible() {
					doReturn(true).when(processor).isResponsibleForEffectedResource(any());
					doReturn(linkBuilder).when(processor).createEffectedResourceLinkBuilder(any());
					var finishedCommand = CommandTestFactory.createBuilder().status(CommandStatus.FINISHED).build();
					var entityModel = EntityModel.of(finishedCommand);

					processor.process(entityModel);

					assertThat(entityModel.getLink(CommandProcessor.REL_EFFECTED_RESOURCE).get().getHref()).isEqualTo("/api/commands/id");
				}

				@Test
				void shouldNotBeAddedIfCommandNotDone() {
					var pendingCommand = CommandTestFactory.createBuilder().status(CommandStatus.PENDING).build();
					var entityModel = EntityModel.of(pendingCommand);

					processor.process(entityModel);

					assertThat(entityModel.getLink(CommandProcessor.REL_EFFECTED_RESOURCE)).isNotPresent();
				}
			}
		}

		@DisplayName("related resource")
		@Nested
		class TestRelatedResource {

			@DisplayName("should verify if responsibility is related to the order")
			@Test
			void shouldCallIsResponsibleForRelatedResource() {
				processor.process(model);

				verify(processor).isResponsibleForRelatedResource(CommandOrder.fromOrder(CommandTestFactory.ORDER));
			}

			@DisplayName("should create link builder for related resource if the responsibility matches")
			@Test
			void shouldCallCreateRelatedResourceLinkBuilder() {
				when(processor.isResponsibleForRelatedResource(any())).thenReturn(true);
				doReturn(linkBuilder).when(processor).createRelatedResourceLinkBuilder(any());

				processor.process(model);

				verify(processor).createRelatedResourceLinkBuilder(command);
			}

			@DisplayName("should NOT create link builder for related resource if the responsibility matches")
			@Test
			void shouldNOTCallCreateRelatedResourceLinkBuilder() {
				when(processor.isResponsibleForRelatedResource(any())).thenReturn(false);

				processor.process(model);

				verify(processor, never()).createRelatedResourceLinkBuilder(command);
			}

			@DisplayName("link")
			@Nested
			class TestRelatedResourceLink {

				@Test
				void shouldBeAddedIfResponsible() {
					doReturn(true).when(processor).isResponsibleForRelatedResource(any());
					doReturn(linkBuilder).when(processor).createRelatedResourceLinkBuilder(any());

					processor.process(model);

					assertThat(model.getLink(CommandProcessor.REL_RELATED_RESOURCE).get().getHref()).isEqualTo("/api/commands/id");
				}
			}
		}
	}
}