/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.forwarding;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.EnumSource;
import org.junit.jupiter.params.provider.EnumSource.Mode;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;
import org.springframework.web.util.UriComponentsBuilder;

import de.ozgcloud.alfa.common.FeatureToggleProperties;
import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserRole;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;

class ForwardingVorgangWithEingangProcessorTest {

	@InjectMocks
	@Spy
	private ForwardingVorgangWithEingangProcessor processor;

	@Mock
	private CurrentUserService userService;
	@Mock
	private FeatureToggleProperties featureToggleProperties;

	@Nested
	class TestProcess {

		private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();

		@Test
		void shouldReturnSameModelOnEmptyEntity() {
			EntityModel<VorgangWithEingang> inputModel = when(mock(EntityModel.class).getContent()).thenReturn(null).getMock();

			var model = processor.process(inputModel);

			assertThat(model).isSameAs(inputModel);
		}

		@Test
		void shouldReturnSameModel() {
			var inputModel = EntityModel.of(vorgang);

			var model = processor.process(inputModel);

			assertThat(model).isSameAs(inputModel);
		}

		@Nested
		class TestForwardByOzgCloudLink {

			@Test
			void shouldCallIsForwardableByOzgCloud() {
				doReturn(false).when(processor).isForwardableByOzgCloud(any());

				processor.process(EntityModel.of(vorgang));

				verify(processor).isForwardableByOzgCloud(vorgang);
			}

			@Nested
			class TestOnIsNotForwardableByOzgCloud {

				@BeforeEach
				void givenIsNotForwardableByOzgCloud() {
					doReturn(false).when(processor).isForwardableByOzgCloud(any());
				}

				@Test
				void shouldNotAddLink() {
					var model = processor.process(EntityModel.of(vorgang));

					assertThat(model.getLink(ForwardingVorgangWithEingangProcessor.REL_FORWARD_BY_OZGCLOUD)).isEmpty();
				}
			}

			@Nested
			class TestOnIsForwardableByOzgCloud {

				@BeforeEach
				void givenIsNotForwardableByOzgCloud() {
					doReturn(true).when(processor).isForwardableByOzgCloud(any());
				}

				@Test
				void shouldAddForwardByOzgCloudLink() {
					var expectedHref = UriComponentsBuilder.fromUriString(ForwardByVorgangCommandController.PATH)
							.buildAndExpand(VorgangHeaderTestFactory.ID, VorgangHeaderTestFactory.VERSION).toString();

					var model = processor.process(EntityModel.of(vorgang));

					assertThat(model.getLink(ForwardingVorgangWithEingangProcessor.REL_FORWARD_BY_OZGCLOUD)).get()
							.extracting(Link::getHref)
							.isEqualTo(expectedHref);
				}
			}
		}

		@Nested
		class TestForwardByEmailLink {

			@Test
			void shouldCallIsForwardableByEmail() {
				doReturn(false).when(processor).isForwardableByEmail();

				processor.process(EntityModel.of(vorgang));

				verify(processor).isForwardableByEmail();
			}

			@Nested
			class TestOnIsNotForwardableByEmail {

				@BeforeEach
				void givenIsNotForwardableByEmail() {
					doReturn(false).when(processor).isForwardableByEmail();
				}

				@Test
				void shouldNotAddLink() {
					var model = processor.process(EntityModel.of(vorgang));

					assertThat(model.getLink(ForwardingVorgangWithEingangProcessor.REL_FORWARD_BY_EMAIL)).isEmpty();
				}
			}

			@Nested
			class TestOnIsForwardableByEmail {

				@BeforeEach
				void givenIsForwardableByEmail() {
					doReturn(true).when(processor).isForwardableByEmail();
				}

				@Test
				void shouldAddLink() {
					var expectedHref = UriComponentsBuilder.fromUriString(ForwardingController.LIST_PATH)
							.queryParam(ForwardingController.PARAM_VORGANG_ID, VorgangHeaderTestFactory.ID).build().toString();

					var model = processor.process(EntityModel.of(vorgang));

					assertThat(model.getLink(ForwardingVorgangWithEingangProcessor.REL_FORWARD_BY_EMAIL)).get()
							.extracting(Link::getHref)
							.isEqualTo(expectedHref);
				}
			}
		}
	}

	@Nested
	class TestIsForwardableByOzgCloud {

		@Nested
		class TestOnFeatureDisabled {

			@BeforeEach
			void givenFeatureDisabled() {
				when(featureToggleProperties.isForwardByOzgCloudEnabled()).thenReturn(false);
			}

			@ParameterizedTest
			@EnumSource
			void shouldReturnFalse(VorgangStatus status) {
				var vorgang = VorgangWithEingangTestFactory.createBuilder().status(status).build();

				var forwardableByOzgCloud = processor.isForwardableByOzgCloud(vorgang);

				assertThat(forwardableByOzgCloud).isFalse();
			}
		}

		@Nested
		class TestOnFeatureEnabled {

			@BeforeEach
			void givenFeatureEnabled() {
				when(featureToggleProperties.isForwardByOzgCloudEnabled()).thenReturn(true);
			}

			@Nested
			class TestOnVorgangNotNeu {
				@ParameterizedTest
				@EnumSource(mode = Mode.EXCLUDE, names = { "NEU" })
				void shouldReturnFalseOnVorgangStatusNotNeu(VorgangStatus status) {
					var vorgang = VorgangWithEingangTestFactory.createBuilder().status(status).build();

					var forwardableByOzgCloud = processor.isForwardableByOzgCloud(vorgang);

					assertThat(forwardableByOzgCloud).isFalse();
				}
			}

			@Nested
			class TestOnVorgangIsNeu {

				private final VorgangWithEingang newVorgang = VorgangWithEingangTestFactory.createBuilder().status(VorgangStatus.NEU).build();

				@Test
				void shouldReturnTrue() {
					var forwardableByOzgCloud = processor.isForwardableByOzgCloud(newVorgang);

					assertThat(forwardableByOzgCloud).isTrue();
				}
			}
		}
	}

	@Nested
	class TestIsForwardableByEmail {

		@Test
		void shouldCallUserServiceForCurrentRole() {
			processor.isForwardableByEmail();

			verify(userService).hasRole(UserRole.EINHEITLICHER_ANSPRECHPARTNER);
		}

		@ParameterizedTest
		@ValueSource(booleans = { true, false })
		void shouldReturnIsEinheitlicherAnsprechpartner(boolean isEinheitlicherAnsprechpartner) {
			when(userService.hasRole(any())).thenReturn(isEinheitlicherAnsprechpartner);

			var isForwardableByEmail = processor.isForwardableByEmail();

			assertThat(isForwardableByEmail).isEqualTo(isEinheitlicherAnsprechpartner);
		}
	}
}
