/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.kommentar;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.Map;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Spy;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandBodyMapper;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.common.command.CommandTestFactory;
import de.ozgcloud.alfa.common.command.LegacyOrder;

class KommentarHistorieProcessorTest {

	@Spy
	private KommentarHistorieProcessor processor;

	@Nested
	class TestProcess {

		private final Command command = CommandTestFactory.create();
		private final Command translatedOnceCommand = CommandTestFactory.create();
		private final Command translatedTwiceCommand = CommandTestFactory.create();

		@BeforeEach
		void mockTranslation() {
			doReturn(translatedOnceCommand).when(processor).translateCreateKommentar(command);
			doReturn(translatedTwiceCommand).when(processor).translateEditKommentar(translatedOnceCommand);
		}

		@Test
		void shouldCallTranslateCreateKommentar() {
			process();

			verify(processor).translateCreateKommentar(command);
		}

		@Test
		void shouldCallTranslateEditKommentar() {
			process();

			verify(processor).translateEditKommentar(translatedOnceCommand);
		}

		@Test
		void shouldReturnStreamOfTranslatedCommand() {
			var commands = process();

			assertThat(commands).containsExactly(translatedTwiceCommand);
		}

		private List<Command> process() {
			return processor.process(Stream.of(command)).toList();
		}
	}

	@Nested
	class TestTranslateCreateKommentar {

		@Nested
		class TestOnNotCreateAttachedItem {

			@Test
			void shouldReturnSameCommand() {
				var command = CommandTestFactory.create();

				var resultCommand = processor.translateCreateKommentar(command);

				assertThat(resultCommand).isSameAs(command);
			}
		}

		@Nested
		class TestOnCreateAttachedItem {

			private Command.CommandBuilder commandBuilder = CommandTestFactory.createBuilder()
					.order(CommandOrder.CREATE_ATTACHED_ITEM.name());

			@Nested
			class TestOnIsCreateKommentar {
				@Test
				void shouldTranslateToCreateKommentar() {
					var commandBody = Map.of(CommandBodyMapper.ITEM_NAME_PROPERTY, Kommentar.class.getSimpleName());
					var command = commandBuilder.body(commandBody).build();
					var expectedCommand = CommandTestFactory.createBuilder()
							.order(LegacyOrder.CREATE_KOMMENTAR)
							.body(commandBody)
							.build();

					var resultCommand = processor.translateCreateKommentar(command);

					assertThat(resultCommand).usingRecursiveComparison().isEqualTo(expectedCommand);
				}
			}

			@Nested
			class TestOnIsNotCreateKommentar {

				@Test
				void shouldReturnSameCommand() {
					var commandBody = Map.of(CommandBodyMapper.ITEM_NAME_PROPERTY, LoremIpsum.getInstance().getWords(1));
					var command = commandBuilder.body(commandBody).build();

					var resultCommand = processor.translateCreateKommentar(command);

					assertThat(resultCommand).isSameAs(command);
				}
			}
		}
	}

	@Nested
	class TestTranslateEditKommentar {

		@Nested
		class TestOnNotUpdateAttachedItem {

			@Test
			void shouldReturnSameCommand() {
				var command = CommandTestFactory.create();

				var resultCommand = processor.translateEditKommentar(command);

				assertThat(resultCommand).isSameAs(command);
			}
		}

		@Nested
		class TestOnUpdateAttachedItem {

			private Command.CommandBuilder commandBuilder = CommandTestFactory.createBuilder()
					.order(CommandOrder.UPDATE_ATTACHED_ITEM.name());

			@Nested
			class TestOnIsEditKommentar {
				@Test
				void shouldTranslateToCreateKommentar() {
					var commandBody = Map.of(CommandBodyMapper.ITEM_NAME_PROPERTY, Kommentar.class.getSimpleName());
					var command = commandBuilder.body(commandBody).build();
					var expectedCommand = CommandTestFactory.createBuilder()
							.order(LegacyOrder.EDIT_KOMMENTAR)
							.body(commandBody)
							.build();

					var resultCommand = processor.translateEditKommentar(command);

					assertThat(resultCommand).usingRecursiveComparison().isEqualTo(expectedCommand);
				}
			}

			@Nested
			class TestOnIsNotEditKommentar {

				@Test
				void shouldReturnSameCommand() {
					var commandBody = Map.of(CommandBodyMapper.ITEM_NAME_PROPERTY, LoremIpsum.getInstance().getWords(1));
					var command = commandBuilder.body(commandBody).build();

					var resultCommand = processor.translateEditKommentar(command);

					assertThat(resultCommand).isSameAs(command);
				}
			}
		}
	}
}
