/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.postfach;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.alfa.common.callcontext.ContextService;
import de.ozgcloud.alfa.common.command.CommandTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.nachrichten.postfach.GrpcFindPostfachMailRequest;
import de.ozgcloud.nachrichten.postfach.GrpcFindPostfachMailsRequest;
import de.ozgcloud.nachrichten.postfach.GrpcGetPostfachConfigRequest;
import de.ozgcloud.nachrichten.postfach.GrpcGetPostfachConfigResponse;
import de.ozgcloud.nachrichten.postfach.GrpcPostfachMail;
import de.ozgcloud.nachrichten.postfach.GrpcResendPostfachMailRequest;
import de.ozgcloud.nachrichten.postfach.PostfachServiceGrpc.PostfachServiceBlockingStub;
import de.ozgcloud.vorgang.grpc.command.GrpcCallContext;

class PostfachMailRemoteServiceTest {

	@InjectMocks // NOSONAR
	private PostfachMailRemoteService service;
	@Mock
	private PostfachServiceBlockingStub serviceStub;
	@Mock
	private PostfachMailMapper postfachNachrichtMapper;
	@Mock
	private PostfachConfigMapper postfachConfigMapper;
	@Mock
	private ContextService contextService;

	private GrpcCallContext callContext = GrpcCallContext.newBuilder().build();

	@Nested
	class TestFindPostfachMail {

		@Captor
		private ArgumentCaptor<GrpcFindPostfachMailRequest> requestCaptor;

		@BeforeEach
		void init() {
			when(contextService.createCallContext()).thenReturn(callContext);
			when(serviceStub.findPostfachMail(any())).thenReturn(GrpcPostfachMailTestFactory.createFindPostfachMailResponse());
		}

		@Test
		void shouldCallStub() {
			service.findById(PostfachMailTestFactory.NACHRICHT_ID);

			verify(serviceStub).findPostfachMail(any(GrpcFindPostfachMailRequest.class));
		}

		@Test
		void shouldCallCallContextService() {
			service.findById(PostfachMailTestFactory.NACHRICHT_ID);

			verify(contextService).createCallContext();
		}

		@Test
		void shouldContainNachrichtId() {
			service.findById(PostfachMailTestFactory.NACHRICHT_ID);

			verify(serviceStub).findPostfachMail(requestCaptor.capture());

			assertThat(requestCaptor.getValue().getNachrichtId()).isEqualTo(PostfachMailTestFactory.ID);
		}

		@Test
		void shouldCallMapper() {
			service.findById(PostfachMailTestFactory.NACHRICHT_ID);

			verify(postfachNachrichtMapper).toPostfachMail(any());
		}
	}

	@Nested
	class TestFindPostfachMails {

		@BeforeEach
		void init() {
			when(contextService.createCallContext()).thenReturn(callContext);
			when(serviceStub.findPostfachMails(any())).thenReturn(GrpcPostfachMailTestFactory.createFindPostfachMailsResponse());
		}

		@Test
		void shouldCallStub() {
			service.findPostfachMails(VorgangHeaderTestFactory.ID);

			verify(serviceStub).findPostfachMails(any(GrpcFindPostfachMailsRequest.class));
		}

		@Test
		void shouldCallCallContextService() {
			service.findPostfachMails(VorgangHeaderTestFactory.ID);

			verify(contextService).createCallContext();
		}

		@Test
		void shouldCallMapper() {
			var result = service.findPostfachMails(VorgangHeaderTestFactory.ID);
			collectStreamElementsToTriggerLazyStream(result);

			verify(postfachNachrichtMapper).toPostfachMail(any(GrpcPostfachMail.class));
		}

		void collectStreamElementsToTriggerLazyStream(Stream<PostfachMail> stream) {
			stream.collect(Collectors.toList());
		}
	}

	@Nested
	class TestResendPostfachMail {

		@BeforeEach
		void init() {
			when(contextService.createCallContext()).thenReturn(callContext);
		}

		@Test
		void shouldCallStub() {
			service.resendPostfachMail(CommandTestFactory.ID, PostfachMailTestFactory.ID);

			verify(serviceStub).resendPostfachMail(any(GrpcResendPostfachMailRequest.class));
		}

		@Test
		void shouldCallContextService() {
			service.resendPostfachMail(CommandTestFactory.ID, PostfachMailTestFactory.ID);

			verify(contextService).createCallContext();
		}
	}

	@Nested
	class TestGetPostfachConfig {

		private final GrpcGetPostfachConfigResponse grpcResponse = GrpcGetPostfachConfigResponseTestFactory.create();
		private final PostfachConfigGroup postfachConfigGroup = PostfachConfigGroupTestFactory.create();

		@BeforeEach
		void mockStub() {
			when(serviceStub.getPostfachConfig(any(GrpcGetPostfachConfigRequest.class))).thenReturn(grpcResponse);
			when(postfachConfigMapper.toPostfachConfigGroup(any(GrpcGetPostfachConfigResponse.class))).thenReturn(postfachConfigGroup);
		}

		@Test
		void shouldCallServiceStub() {
			service.getPostfachConfig();

			verify(serviceStub).getPostfachConfig(any(GrpcGetPostfachConfigRequest.class));
		}

		@Test
		void shouldCallMapper() {
			service.getPostfachConfig();

			verify(postfachConfigMapper).toPostfachConfigGroup(grpcResponse);
		}

		@Test
		void shouldReturnPostfachConfigs() {
			var result = service.getPostfachConfig();

			assertThat(result).isEqualTo(postfachConfigGroup);
		}
	}
}