/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.resource;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;

import org.hamcrest.core.StringEndsWith;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;

import de.ozgcloud.alfa.common.AlfaTestUtils;
import lombok.SneakyThrows;

@AutoConfigureMockMvc
@SpringBootTest
@WithMockUser
class OzgcloudResourceControllerITCase {

	@Autowired
	private MockMvc mockMvc;

	@Nested
	class TestGetOzgcloudResource {

		private static final String VORGANG_ID = AlfaTestUtils.createMongoDbObjectId();
		private static final String VALID_URI = "dns:///dummy.service:123/vorgangs/" + VORGANG_ID;
		private static final String INVALID_URI = "dns:///dummy.service:123/" + VORGANG_ID;

		@Test
		void shouldReturnStatusOk() throws Exception {
			var response = doRequest("dns:///test.de/vorgangs/" + VORGANG_ID);

			response.andExpect(status().isOk());
		}

		@Test
		void shouldHaveVorgangLink() throws Exception {
			var response = doRequest(VALID_URI);

			response.andExpect(jsonPath("$._links.vorgang.href").value(StringEndsWith.endsWith("/api/vorgangs/" + VORGANG_ID)));
		}

		@Test
		void shouldHaveSelfLink() throws Exception {
			var encodedUri = URLEncoder.encode(VALID_URI, StandardCharsets.UTF_8);

			var response = doRequest(VALID_URI);

			response.andExpect(jsonPath("$._links.self.href").value(StringEndsWith.endsWith(OzgcloudResourceController.PATH + "?uri=" + encodedUri)));
		}

		@Test
		void shouldReturnStatusNotFound() throws Exception {
			var response = doRequest(INVALID_URI);

			response.andExpect(status().isNotFound());
		}

		@Test
		void shouldReturnBadRequestOnNoRequestParam() throws Exception {
			var response = doRequestWithQueryString("");

			response.andExpect(status().isBadRequest());
		}

		@Test
		void shouldReturnBadRequestOnEmptyUri() throws Exception {
			var response = doRequestWithQueryString("?uri=");

			response.andExpect(status().isBadRequest());
		}

		@SneakyThrows
		private ResultActions doRequest(String uri) {
			return doRequestWithQueryString("?uri=" + uri);
		}

		@SneakyThrows
		private ResultActions doRequestWithQueryString(String queryString) {
			return mockMvc.perform(get(OzgcloudResourceController.PATH + queryString));
		}
	}
}
